@@ -1,13 +1,11 @@
 from datetime import datetime, timezone
 import json
 import requests
-import time
-from typing import Callable, Union
-import warnings
-import websocket
+from typing import Union
 
-from .event import EventType
 from ._logging import _logger
+from .event import EventType
+from .flairs import Flair, flair_converter
 from .funcs import threaded
 from .message import (
     Message,
@@ -18,6 +16,7 @@ from .message import (
     VoteMessage,
 )
 from .user import User
+from .wsbase import WSBase
 from .errors import (
     AccountTooYoung,
     Banned,
@@ -33,9 +32,14 @@ from .errors import (
 )
 
 
-class DGGChat:
-    WSS = "wss://chat.destiny.gg/ws"
-    URL = "https://www.destiny.gg"
+class DGGChat(WSBase):
+    _CONFIG = {
+        "wss": "wss://chat.destiny.gg/ws",
+        "wss-origin": "https://www.destiny.gg",
+        "baseurl": "https://www.destiny.gg",
+        "endpoints": {"user": "/api/chat/me", "userinfo": "/api/userinfo"},
+        "flairs": "https://cdn.destiny.gg/flairs/flairs.json",
+    }
 
     def __init__(
         self,
@@ -44,16 +48,9 @@ class DGGChat:
         *,
         sid: str = None,
         rememberme: str = None,
+        config: Union[str, dict[str, dict]] = None,
         **kwargs,
     ):
-        if (
-            "username" in kwargs
-        ):  # remove this after the first update on or after 12 March 2023
-            message = "The 'username' variable has been deprecated as of 0.9.0"
-            warnings.warn(message, DeprecationWarning, stacklevel=2)
-            _logger.warning(message)
-        self.authenticated = False
-        self.wss = wss or self.WSS
         cookie = None
         self.username = None
         if auth_token:
@@ -62,17 +59,9 @@ class DGGChat:
         elif sid:
             cookie = f"sid={sid}" + (f";rememberme={rememberme}" if rememberme else "")
             self.username = self._get_username_from_sid(cookie)
-
-        self.ws = websocket.WebSocketApp(
-            self.wss,
-            cookie=cookie,
-            on_open=self._on_open,
-            on_message=self._on_message,
-            on_error=self._on_error,
-            on_close=self._on_close,
-        )
-        self._connected = False
-        self._events = {}
+        super().__init__(wss, cookie, config=config)
+        self._flairs = flair_converter(self.config["flairs"])
+        self.authenticated = False
         self._users = {}
 
     def __repr__(self):
@@ -91,8 +80,14 @@ class DGGChat:
             epoch // 1000, tz=timezone.utc
         )  # dgg sends milliseconds
 
+    def _convert_flairs(self, data: dict) -> Union[list[Flair], None]:
+        if "features" in data:
+            return [self._flairs[flair] for flair in data["features"]]
+
     def _get_username_from_token(self, auth_token: str) -> Union[str, None]:
-        r = requests.get(f"{self.URL}/api/userinfo?token={auth_token}")
+        r = requests.get(
+            f"{self.config['baseurl']}{self.config['endpoints']['userinfo']}?token={auth_token}"
+        )
         resp = r.json()
         if "username" in resp:
             return resp["username"].lower()
@@ -103,7 +98,10 @@ class DGGChat:
 
     def _get_username_from_sid(self, cookie: str) -> Union[str, None]:
         headers = {"cookie": cookie}
-        r = requests.get(f"{self.URL}/api/chat/me", headers=headers)
+        r = requests.get(
+            self.config["baseurl"] + self.config["endpoints"]["user"],
+            headers=headers,
+        )
         resp = r.json()
         if r.status_code == 200:
             resp = r.json()
@@ -120,6 +118,9 @@ class DGGChat:
     def get_user(self, username: str) -> Union[User, None]:
         return self._users.get(username.lower())
 
+    def get_flair(self, name: str) -> Flair:
+        return self._flairs[name]
+
     _err_dict = {
         "banned": Banned,
         "duplicate": DuplicateMessage,
@@ -143,18 +144,11 @@ class DGGChat:
                 event_type,
                 data["nick"],
                 self._dggtime_to_dt(data["createdDate"]),
-                data["features"],
+                self._convert_flairs(data),
                 self._dggepoch_to_dt(data["timestamp"]),
                 data["data"],
                 data["duration"],
             )
-        elif event_type == EventType.BROADCAST:
-            msg = Message(
-                self,
-                event_type,
-                timestamp=self._dggepoch_to_dt(data["timestamp"]),
-                data=data["data"],
-            )
         elif event_type == EventType.PRIVMSG:
             msg = PrivateMessage(
                 self,
@@ -182,7 +176,7 @@ class DGGChat:
                 event_type,
                 data["nick"],
                 self._dggtime_to_dt(data["createdDate"]),
-                data["features"],
+                self._convert_flairs(data),
                 self._dggepoch_to_dt(data["timestamp"]),
                 data.get("data"),
                 data["uuid"],
@@ -211,39 +205,29 @@ class DGGChat:
                 data["nick"],
                 vote=data["vote"],
             )
-        elif event_type in (
-            EventType.MESSAGE,
-            EventType.UNMUTE,
-            EventType.BAN,
-            EventType.UNBAN,
-            EventType.SUBONLY,
-            EventType.BROADCAST,
-            EventType.JOIN,
-            EventType.QUIT,
-            EventType.REFRESH,
-        ):
+        else:
             msg = Message(
                 self,
                 event_type,
-                data["nick"],
-                self._dggtime_to_dt(data["createdDate"]),
-                data["features"],
+                data.get("nick"),
+                self._dggtime_to_dt(data["createdDate"])
+                if "createdDate" in data
+                else None,
+                self._convert_flairs(data),
                 self._dggepoch_to_dt(data["timestamp"]),
                 data.get("data"),
             )
-        else:
-            _logger.warning(f"Unknown event type: {event_type} {data}")
-            return
-        func_name = f"on_{event_type.lower()}"
-        if hasattr(self, func_name):
-            f = getattr(self, func_name)
-            f(msg)
-        else:
-            _logger.warning(f"Function '{func_name}' not found.")
+
+        if event_type == EventType.QUIT:
+            self._users.pop(msg.nick_lower, None)
+        elif event_type == EventType.JOIN:
+            self._users[msg.nick_lower] = User(msg.nick, msg.createdDate, msg.features)
+
+        self.on_event(event_type.lower(), msg)
         if event_type in (EventType.MESSAGE, EventType.PRIVMSG) and self.is_mentioned(
             msg
         ):
-            self.on_mention(msg)
+            self.on_event("mention", msg)
 
     def _on_open(self, ws):
         _logger.info(
@@ -253,25 +237,6 @@ class DGGChat:
         )
         self._connected = True
 
-    def _on_close(self, ws, *_):
-        _logger.debug(f"Connection closed.")
-        self._connected = False
-
-    def _on_error(self, ws, error):
-        _logger.error(error)
-
-    def event(self, event_name: str = None):
-        """Decorator to run function when the specified event occurs."""
-
-        def decorator(func: Callable):
-            event = event_name or func.__name__
-            if event not in self._events:
-                self._events[event] = []
-            self._events[event].append(func)
-            return func
-
-        return decorator
-
     def mention(self):
         """Decorator to run function on mentions. Shortcut for event('on_mention')."""
         return self.event("on_mention")
@@ -281,50 +246,18 @@ class DGGChat:
             False if self.username is None else (self.username in msg.data.casefold())
         )
 
-    @threaded
-    def on_mention(self, msg):
-        """Do stuff when mentioned."""
-        for func in self._events.get("on_mention", tuple()):
-            func(msg)
-
-    @threaded
-    def on_msg(self, msg: Message):
-        """Do stuff when a MSG is received."""
-        for func in self._events.get("on_msg", tuple()):
-            func(msg)
-
     @threaded
     def on_names(self, connection_count: int, users: list):
         """Do stuff when the NAMES message is received upon connecting to chat."""
         self._users = {
             user["nick"].lower(): User(
-                user["nick"], self._dggtime_to_dt(user["createdDate"]), user["features"]
+                user["nick"],
+                self._dggtime_to_dt(user["createdDate"]),
+                user["features"],
             )
             for user in users
         }
-        for func in self._events.get("on_names", tuple()):
-            func(connection_count, users)
-
-    @threaded
-    def on_pin(self, msg: PinnedMessage):
-        """Do stuff when a PIN is received."""
-        for func in self._events.get("on_pin", tuple()):
-            func(msg)
-
-    @threaded
-    def on_privmsg(self, msg: PrivateMessage):
-        """Do stuff when a PRIVMSG is received."""
-        for func in self._events.get("on_privmsg", tuple()):
-            func(msg)
-
-    def run(self, origin: str = None):
-        self.ws.run_forever(origin=origin or self.URL)
-
-    def run_forever(self, origin: str = None, sleep: int = 2):
-        """Runs the client forever by automatically reconnecting the websocket."""
-        while True:
-            self.run(origin=origin or self.URL)
-            time.sleep(sleep)
+        self.on_event("names", connection_count, users)
 
     @threaded
     def send(self, msg: str):
@@ -343,77 +276,3 @@ class DGGChat:
         """Participate in a chat poll."""
         payload = {"vote": str(vote)}
         self.ws.send(f"CASTVOTE {json.dumps(payload)}")
-
-    @threaded
-    def on_broadcast(self, msg):
-        """Do stuff when a BROADCAST is received."""
-        for func in self._events.get("on_broadcast", tuple()):
-            func(msg)
-
-    @threaded
-    def on_join(self, msg: Message):
-        """Do stuff when chatter joins."""
-        self._users[msg.nick_lower] = User(msg.nick, msg.createdDate, msg.features)
-        for func in self._events.get("on_join", tuple()):
-            func(msg)
-
-    @threaded
-    def on_quit(self, msg):
-        """Do stuff when chatter joins."""
-        self._users.pop(msg.nick_lower)
-        for func in self._events.get("on_quit", tuple()):
-            func(msg)
-
-    @threaded
-    def on_ban(self, msg: Message):
-        """Do stuff when a chatter is banned."""
-        for func in self._events.get("on_ban", tuple()):
-            func(msg)
-
-    @threaded
-    def on_unban(self, msg: Message):
-        """Do stuff when a chatter is unbanned."""
-        for func in self._events.get("on_unban", tuple()):
-            func(msg)
-
-    @threaded
-    def on_mute(self, msg: MuteMessage):
-        """Do stuff when a chatter is muted."""
-        for func in self._events.get("on_mute", tuple()):
-            func(msg)
-
-    @threaded
-    def on_unmute(self, msg: Message):
-        """Do stuff when a chatter is unmuted."""
-        for func in self._events.get("on_unmute", tuple()):
-            func(msg)
-
-    @threaded
-    def on_subonly(self, msg: Message):
-        """Do stuff when sub-only is turned on/off."""
-        for func in self._events.get("on_subonly", tuple()):
-            func(msg)
-
-    @threaded
-    def on_refresh(self, msg: Message):
-        """Do stuff when refreshed."""
-        for func in self._events.get("on_refresh", tuple()):
-            func(msg)
-
-    @threaded
-    def on_pollstart(self, msg: PollMessage):
-        """Do stuff when a poll is started."""
-        for func in self._events.get("on_pollstart", tuple()):
-            func(msg)
-
-    @threaded
-    def on_pollstop(self, msg: PollMessage):
-        """Do stuff when a poll is ended."""
-        for func in self._events.get("on_pollstop", tuple()):
-            func(msg)
-
-    @threaded
-    def on_votecast(self, msg: VoteMessage):
-        """Do stuff when a vote is cast in a poll."""
-        for func in self._events.get("on_votecast", tuple()):
-            func(msg)
